package Anarres::Mud::Driver::Program::Type;

use strict;
use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS);
use Exporter;
use Carp qw(:DEFAULT cluck);
use Data::Dumper;

BEGIN {
	no strict qw(refs);

	my %types = (
		T_VOID		=> "V",
		T_NIL		=> "N",
		T_MIXED		=> "_",
		T_BOOL		=> "b",
		T_CLOSURE	=> "f",
		T_INTEGER	=> "i",
		T_MAPPING	=> "m",
		T_OBJECT	=> "o",
		T_STRING	=> "s",
		T_ARRAY		=> "P_",
			);
	my $package = __PACKAGE__;

	foreach (keys %types) {
		eval qq{
			*{"$package\::$_"} = sub () {
				my \$self = "$types{$_}";
				return bless \\\$self, "$package";
			};
		}; die $@ if $@;
	}

	@ISA = qw(Exporter);
	@EXPORT_OK = keys %types;
	%EXPORT_TAGS = ( types => \@EXPORT_OK );
}

sub struct {
	my ($class, @fields) = @_;			# @fields are Types
	my $self = "S{" . join(",", map { $$_ } @fields) . "}";
	return bless \$self, $class;
}

sub equals {
	my ($self, $t) = @_;
	return $$self eq $$t;
}

sub classp { return ${$_[0]} =~ /^S/; }
sub arrayp { return ${$_[0]} =~ /^P/; }

sub pointer {
	my ($self, $stars) = @_;	# $type is a Type
	$stars or $stars = 1;
	my $type = ("P" x $stars) . $$self;
	return bless \$type, ref($self);
}

sub deref {
	my ($self) = @_;
	return "_" if $$self eq '_';
	return undef unless $$self =~ /^P/;
	my $out = substr($$self, 1);
	return bless \$out, ref($self);
}

sub dump { return ${$_[0]}; }

1;
